/*
 * COPYRIGHT: FREQUENTIS AG. All rights reserved.
 *            Registered with Commercial Court Vienna,
 *            reg.no. FN 72.115b.
 */
package com.frequentis.ps.xap.lease;

import com.gigaspaces.cluster.activeelection.SpaceMode;
import com.j_spaces.core.client.EntryArrivedRemoteEvent;
import java.time.Duration;
import org.openspaces.core.GigaSpace;
import org.openspaces.core.space.mode.AfterSpaceModeChangeEvent;
import org.openspaces.core.space.mode.SpaceAfterBackupListener;
import org.openspaces.core.space.mode.SpaceAfterPrimaryListener;
import org.openspaces.events.adapter.SpaceDataEvent;
import org.openspaces.events.notify.SimpleNotifyContainerConfigurer;
import org.openspaces.events.notify.SimpleNotifyEventListenerContainer;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;

@Service
public class LeaseBootstrapper implements SpaceAfterPrimaryListener, SpaceAfterBackupListener, DisposableBean {
    private final GigaSpace space;
    private final PlatformTransactionManager txManager;
    private SpaceMode currentSpaceMode = SpaceMode.NONE;
    private SimpleNotifyEventListenerContainer notifyContainer = null;

    @Autowired
    public LeaseBootstrapper(final @Qualifier("gigaSpace") GigaSpace space, final PlatformTransactionManager txManager) {
        this.space = space;
        this.txManager = txManager;
    }

    @Override
    public void onAfterPrimary(final AfterSpaceModeChangeEvent afterSpaceModeChangeEvent) {
        currentSpaceMode = afterSpaceModeChangeEvent.getSpaceMode();
        System.out.println("*** Register for lease expiration in PRIMARY");
        registerLeaseExpiration();
    }

    @Override
    public void onAfterBackup(final AfterSpaceModeChangeEvent afterSpaceModeChangeEvent) {
        currentSpaceMode = afterSpaceModeChangeEvent.getSpaceMode();
        System.out.println("*** Register for lease expiration in BACKUP");
        registerLeaseExpiration();
    }

    private void registerLeaseExpiration() {
        if (notifyContainer == null) {
            notifyContainer = new SimpleNotifyContainerConfigurer(this.space)
                    .template(ExpirationObject.template())
                    .transactionManager(this.txManager)
                    .eventListenerAnnotation(this)
                    .notifyLeaseExpire(true)
                    .replicateNotifyTemplate(true)
                    .fifo(true)
                    .notifyContainer();

            notifyContainer.start();
        }
    }

    @Override
    public void destroy() throws Exception {
        System.out.println("*** Destroy notify container in " + currentSpaceMode);
        notifyContainer.destroy();
        notifyContainer = null;

        // artificially extend the shutdown windows
        // without the window is of course much smaller but still there
        // if a lease expires during this time, the event is lost
        Thread.sleep(Duration.ofSeconds(7).toMillis());
    }

    /**
     * Called by the space notify container as event occurs.
     */
    @SpaceDataEvent
    @SuppressWarnings("all")
    private void leaseExpiredEvent(final ExpirationObject data, final GigaSpace gigaSpace, final TransactionStatus txStatus, final EntryArrivedRemoteEvent source) {
        System.out.println(" ** Expired event in Mode " + currentSpaceMode + ", Nr: " + data.getSequenceNumber());
    }
}